"""music.py

Part of the Tundra project.
(C) 2006 Rebuild Software
http://rebui.ld.net.au

This file contains handlers for the sound subsystem.
"""

import globals as g

from pygame.locals import *
import pygame 

import os, sys
import random
import xml.sax
import xml.sax.writer
from xml.sax.handler import ContentHandler

# the following file uses simple XML for playlisting
# this will make it easier later on to have an in-game playlist editor
# until someone complains and demands M3U support, this is how it will stay :)
# (god I love being a one man team, it's so empowering)

checkPath = lambda p: os.path.isfile(g.RESOURCES_PATH+"music/"+p) and (p.split(".")[-1].lower() in ["mp3","ogg","mod","xm","s3m","it"])


class Music:
    def __init__(self):
        self.name = "Dehydrated Playlist! Just add water and microwave!"
        # ok, here's how it all works:
        # self.collection is a dictionary containing entries corresponding to level collections.
        # Each of these entries contain a list of file paths.         
        
        # self.collection["all"]     - all tracks in the playlist are added to this list
        # self.special          - NOT a list, but a dictionary used to catalogue special level 
        #                         tunes that are referred to in each level xml by "musicKeyword=[key]"
        #                         and are referenced in music XML files by <special src=... keyword=...>"
        self.collection = {"all":[]}
        self.special = {}
        self.currentColl = "all"
        self.index = -1
        self.currentPath = None
        self.random = True
        self.noColls = False # used to determine whether to search self.collection or default to "all"

    def setCollection(self, name):
        if not self.noColls and self.collection.has_key(name) and (len(self.collection[name])>0):
            self.currentColl = name
            self.index = -1
            self.random = g.RANDOM_MUSIC   # this is set from the Tundra options menu
        else:
            self.currentColl = "all"
            self.index = -1
            self.random = True

    def next(self, keyword=None):
        if not g.ENABLE_MUSIC or len(self.collection[self.currentColl])==0:
            return # don't bother trying this, you'll just hurt yourself
        if self.special.has_key(keyword):
            self.currentPath = self.special[keyword]
        elif self.random:
            self.index = random.randint(0,len(self.collection[self.currentColl])-1)
            self.currentPath = self.collection[self.currentColl][self.index]        
        else:
            self.index += 1
            self.index %= len(self.collection[self.currentColl])
            self.currentPath = self.collection[self.currentColl][self.index]
        return        

    def fadeout(self):
        try:
            if pygame.mixer.music.get_busy():
                pygame.mixer.music.fadeout(1500)
        except pygame.error:
            pass        # mixer hasn't initialized        
        return

    def play(self):
        if g.ENABLE_MUSIC and len(self.collection[self.currentColl])>0: # don't bother if music is turned off
            if self.index == -1: # should never happen
                self.next()
            pygame.mixer.music.set_volume(1.0)
            pygame.mixer.music.load(self.currentPath)
            pygame.mixer.music.play(-1)
        return


class PlaylistHandler(ContentHandler):
    def __init__(self):
        self.result = Music()
        self.isCollArea = False
        self.collName = None

    def startElement(self, name, attrs):
        if name == "playlist":
            self.result.name = str(attrs.get("name", "Playlist"))

        elif (name == "collection") & attrs.has_key("name"):
            self.isCollArea = True
            self.collName = str(attrs["name"])
            self.result.collection[self.collName] = []

        elif (name == "track") & attrs.has_key("src"):
            srcpath = str(attrs["src"])
            if checkPath(srcpath):            
                if self.isCollArea:
                    self.result.collection[self.collName].append(g.RESOURCES_PATH+"music/"+srcpath)
                self.result.collection["all"].append(g.RESOURCES_PATH+"music/"+srcpath)

        elif (name == "special") & attrs.has_key("src") & attrs.has_key("keyword"):
            srcpath = str(attrs["src"])
            if checkPath(srcpath):  
                self.result.special[str(attrs["keyword"])] = g.RESOURCES_PATH+"music/"+srcpath

    def endElement(self, name):
        if name == "collection":
            self.isCollArea = False


def importPlaylistFromXML(filename):
    parser = xml.sax.make_parser()
    handler = PlaylistHandler()
    parser.setContentHandler(handler)
    parser.parse(open(filename))
    return handler.result

def loadPlaylist(filename):
    if not filename == "random":
        mpath = g.RESOURCES_PATH+"/music/"+filename+".xml"
        if os.path.isfile(mpath):
            return importPlaylistFromXML(mpath)
        elif os.path.isfile(mpath+".gz"):
            return importPlaylistFromXML(mpath+".gz")
    return automaticRandomPlaylist()


def automaticRandomPlaylist():
    result = Music()   
    for n in [(g.RESOURCES_PATH+"music/"+x) for x in os.listdir(g.RESOURCES_PATH+"music/") if checkPath(x)]:
        result.collection["all"].append(n)
    result.currentColl = "all"
    result.random = True
    result.noColls = True
    return result
